package at.ac.uibk.gitsearch.service.dto;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

public class SearchResultDTO {
	
	public static class PluginActionInfo {
		String plugin;
		String action;
		String commandName;
		
		public PluginActionInfo(String plugin, String action, String commandName) {
			super();
			this.plugin = plugin;
			this.action = action;
			this.commandName = commandName;
		}
		
		public PluginActionInfo() {
			// JSON
		}
		/**
		 * @return the plugin
		 */
		public String getPlugin() {
			return plugin;
		}
		/**
		 * @param plugin the plugin to set
		 */
		public void setPlugin(String plugin) {
			this.plugin = plugin;
		}
		/**
		 * @return the action
		 */
		public String getAction() {
			return action;
		}
		/**
		 * @param action the action to set
		 */
		public void setAction(String action) {
			this.action = action;
		}
		/**
		 * @return the commandName
		 */
		public String getCommandName() {
			return commandName;
		}
		/**
		 * @param commandName the commandName to set
		 */
		public void setCommandName(String commandName) {
			this.commandName = commandName;
		}
		
		
	}


    private GitProject project;
    private MetadataFile file;
    private UserProvidedMetadataDTO metadata;
    
    private int ranking5;
	private List<PluginActionInfo> supportedActions = new ArrayList<>(2);

	public SearchResultDTO() {
        // default constructor
    }

    /**
     * clone constructor
     *
     * @param toClone
     */
    public SearchResultDTO(SearchResultDTO toClone) {
        super();
        this.project = toClone.project;
        this.file = toClone.file;
        this.metadata = toClone.metadata;
    }

    public GitProject getProject() {
        return project;
    }

    public void setProject(GitProject project) {
        this.project = project;
    }

    public MetadataFile getFile() {
        return file;
    }

    public void setFile(MetadataFile file) {
        this.file = file;
    }

    public UserProvidedMetadataDTO getMetadata() {
        return metadata;
    }

    public void setMetadata(UserProvidedMetadataDTO metadata) {
        this.metadata = metadata;
    }

    @Override
    public String toString() {
        return "SearchResultDTO : { project: " + this.project
            + ", file: " + this.file
            + ", metadata: " + this.metadata
            + " }";
    }

    @Override
    public final int hashCode() {
        int prime = 31;
        int result=0;
        if(this.project!=null)
          result = this.project.hashCode();
        if(this.file!=null)
        	result = prime * result + this.file.hashCode();
        if(this.metadata!=null)
        	result = prime * result + this.metadata.hashCode();
        return result;
    }

    @Override
    public final boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (!(obj instanceof SearchResultDTO)) {
            return false;
        }
        SearchResultDTO other = (SearchResultDTO) obj;
        return Objects.equals(this.project, other.project)
            && Objects.equals(this.file, other.file)
            && Objects.equals(this.metadata, other.metadata);
    }

    public static class GitProject {
        private int project_id;
        private String project_name;
        private String namespace;
        private String main_group;
        private String sub_group;
        private String url;
        private Date last_activity_at;

        public int getProject_id() {
            return project_id;
        }

        public Date getLast_activity_at() {
            return last_activity_at;
        }

        public void setLast_activity_at(Date last_activity_at) {
            this.last_activity_at = last_activity_at;
        }

        public void setProject_id(int project_id) {
            this.project_id = project_id;
        }

        public String getProject_name() {
            return project_name;
        }

        public void setProject_name(String project_name) {
            this.project_name = project_name;
        }

        public String getNamespace() {
            return namespace;
        }

        public void setNamespace(String namespace) {
            this.namespace = namespace;
        }

        public String getMain_group() {
            return main_group;
        }

        public void setMain_group(String main_group) {
            this.main_group = main_group;
        }

        public String getSub_group() {
            return sub_group;
        }

        public void setSub_group(String sub_group) {
            this.sub_group = sub_group;
        }

        public String getUrl() {
            return url;
        }

        public void setUrl(String url) {
            this.url = url;
        }

        public int getProjectId() {
            return project_id;
        }

        public void setProjectId(int project_id) {
            this.project_id = project_id;
        }

        @Override
        public final int hashCode() {
            return Integer.hashCode(project_id);
        }

        @Override
        public final boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (!(obj instanceof GitProject)) {
                return false;
            }
            GitProject other = (GitProject) obj;
            return this.project_id == other.project_id;
        }

        @Override
        public String toString() {
            return  "Project: { project_id: " + project_id
                + ", project_name: " + project_name
                + ", namespace: " + namespace
                + ", main_group: " + main_group
                + ", sub_group: " + sub_group
                + ", url: " + url
                + " }";
        }
    }
    
    /**
	 * @return the ranking5
	 */
	public int getRanking5() {
		return ranking5;
	}

	/**
	 * @param ranking5 the ranking5 to set
	 */
	public void setRanking5(int ranking5) {
		this.ranking5 = ranking5;
	}

	/**
	 * @return the supportedActions
	 */
	public List<PluginActionInfo> getSupportedActions() {
		return supportedActions;
	}

	/**
	 * @param supportedActions the supportedActions to set
	 */
	public void setSupportedActions(List<PluginActionInfo> supportedActions) {
		this.supportedActions = supportedActions;
	}

    public static class MetadataFile {
        private String filename;
        private String path;
        private String commit_id;
        private Date indexing_date;

        public String getFilename() {
            return filename;
        }

        public void setFilename(String filename) {
            this.filename = filename;
        }

        public String getPath() {
            return path;
        }

        public void setPath(String path) {
            this.path = path;
        }

        public String getCommit_id() {
            return commit_id;
        }

        public void setCommit_id(String commit_id) {
            this.commit_id = commit_id;
        }

        public Date getIndexing_date() {
            return indexing_date;
        }

        public void setIndexing_date(Date indexing_date) {
            this.indexing_date = indexing_date;
        }
        
        @Override
        public final int hashCode() {
            final int prime = 31;
            int result = this.path.hashCode();
            result = prime * result + this.commit_id.hashCode();
            return result;
        }

        @Override
        public final boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (!(obj instanceof MetadataFile)) {
                return false;
            }
            MetadataFile other = (MetadataFile) obj;
            return Objects.equals(this.path, other.path)
                && Objects.equals(this.commit_id, other.commit_id);
        }

        @Override
        public String toString() {
            return "MetadataFile: { filename: " + this.filename
                + ", path: " + this.path
                + ", commit_id: " + this.commit_id
                + ", indexing_date: " + this.indexing_date
                + " }";
        }
    }
}
